import Image = Phaser.GameObjects.Image;
import BaseScene from '../../scenes/base.scene';

export class BaseButton extends Phaser.GameObjects.Container {
  frame: string;
  buttonHeight: number;
  buttonWidth: number;
  button: Image;

  isShowed: boolean;

  callback;
  callbackInterval;
  callbackTimeout;

  constructor(scene: BaseScene, x: number, y: number, width: number, height: number, frame: string, callback = () => {}, repeatingFire = false) {
    super(scene, x, y);

    this.callback = callback;
    this.frame = frame;
    this.buttonWidth = width;
    this.buttonHeight = height;
    this.button = new Image(scene, 0, 0, 'ui', this.frame + '.png');
    this.button.setDisplaySize(width, height).setInteractive({useHandCursor: true});
    if (repeatingFire) {
      const restore = () => {
        this.enterButtonNormalState();
        clearTimeout(this.callbackTimeout);
        clearInterval(this.callbackInterval);
      };
      this.button
        .on('pointerdown', () => {
          this.enterButtonActiveState();
          this.clickCallback();
          clearTimeout(this.callbackTimeout);
          clearInterval(this.callbackInterval);
          this.callbackTimeout = setTimeout(() => {
            this.callbackInterval = setInterval(() => this.clickCallback(), 175);
          }, 150);
        })
        .on('pointerout', restore)
        .on('pointerup', restore);
    } else {
      this.button
        .on('pointerdown', () => this.enterButtonActiveState())
        .on('pointerout', () => this.enterButtonNormalState())
        .on('pointerup', () => {
          this.enterButtonNormalState();
          this.clickCallback();
        });
    }
  }

  enterButtonNormalState() {
    this.button.setFrame(this.frame + '.png');
  }

  enterButtonActiveState() {
    this.button.setFrame(this.frame + '_pressed.png');
  }

  clickCallback() {
    this.callback();
  }

  show(duration = 1000) {
    this.isShowed = true;
    this.scene.tweens.add({
      targets: this,
      alpha: {from: 0, to: 1},
      ease: 'Sine.easeOut',
      duration: duration,
    });
  }

  hide(duration = 1000) {
    if (!this.isShowed) {return;}
    this.scene.tweens.add({
      targets: this,
      alpha: {from: 1, to: 0},
      ease: 'Sine.easeOut',
      duration: duration,
      onComplete: () => {
        this.isShowed = false;
      }
    });
  }
}
