import Image = Phaser.GameObjects.Image;
import Text = Phaser.GameObjects.Text;

export class BaseProgressBar extends Phaser.GameObjects.Container {
  width;
  height;
  edgeWidth;

  edgeValue: number;
  leftEdgeValue: number;
  rightEdgeValue: number;
  middleValue: number;

  edges: Image[] = [];
  filler: Image[] = [];
  text: Text;

  origins = [0, 0, 1];
  color: string;
  progress: number = 1;
  showText: boolean;

  autoColor = true;
  colorMap = {
    'Red' : 0.175,
    'Yellow': 0.4,
    'Green': 1,
  };

  constructor(scene, x, y, width, height, color: 'Green'|'Yellow'|'Red'|'Blue' = 'Green', origins, showText = false) {
    super(scene, x, y);

    this.width = width;
    this.height = height;
    this.color = color;
    this.origins = origins;
    this.showText = showText;
    this.edgeWidth = this.height / 2;

    let image: Image;
    image = new Image(scene, 0, 0, 'ui', `barBack_horizontalLeft.png`);
    image.setOrigin(this.origins[0], 0);
    this.edges.push(image);
    image = new Image(scene, 0, 0, 'ui', `barBack_horizontalMid.png`);
    image.setOrigin(this.origins[1], 0);
    this.edges.push(image);
    image = new Image(scene, 0, 0, 'ui', `barBack_horizontalRight.png`);
    image.setOrigin(this.origins[2], 0);
    this.edges.push(image);

    image = new Image(scene, 0, 0, 'ui', `bar${color}_horizontalLeft.png`);
    image.setOrigin(this.origins[0], 0);
    this.filler.push(image);
    image = new Image(scene, 0, 0, 'ui', `bar${color}_horizontalMid.png`);
    image.setOrigin(this.origins[1], 0);
    this.filler.push(image);
    image = new Image(scene, 0, 0, 'ui', `bar${color}_horizontalRight.png`);
    image.setOrigin(this.origins[2], 0);
    this.filler.push(image);

    this.add(this.edges);
    this.add(this.filler);

    if (this.showText) {
      const style = {fontSize: Math.floor(2 * height / 5) + 'px', fontFamily: '"Architects Daughter"', color: 'black'};
      this.text = new Text(scene, 0, Math.ceil(this.height / 2), '100%', style).setOrigin(0.5);
      this.add(this.text);
    }

    this.setWidth(width);
  }

  setColor(color) {
    if (this.color == color) {
      return;
    }
    this.color = color;
    this.filler[0].setFrame(`bar${color}_horizontalLeft.png`);
    this.filler[1].setFrame(`bar${color}_horizontalMid.png`);
    this.filler[2].setFrame(`bar${color}_horizontalRight.png`);
  }

  checkColor() {
    for (const color in this.colorMap) {
      if (this.progress <= this.colorMap[color]) {
        this.setColor(color);
        break;
      }
    }
  }

  setProgress(progress: number) {
    this.progress = progress;
    if (this.autoColor) {
      this.checkColor();
    }
    if (this.showText) {
      this.text.setText(Math.floor(this.progress * 100) + '%');
    }
  }

  changeProgress(delta) {
    this.setProgress(Math.min(Math.max(this.progress + delta, 0), 1));
  }

  setWidth(width: number) {

  }
}
