import { BaseProgressBar } from './base';

const tileHeight = 18;
const edgeWidth = 9;

export class RoundedProgressBar extends BaseProgressBar {
  realEdgeWidth;
  middleWidth;
  cut;

  left;
  right;

  constructor(scene, x, y, width, height, color: 'Green'|'Yellow'|'Red'|'Blue' = 'Green', showText = false) {
    super(scene, x, y, width, height, color, [0, 0, 1], showText);
  }

  setProgress(progress: number|null) {
    if (progress === this.progress) {
      return;
    }
    if (progress === null) {
      progress = this.progress;
    }
    super.setProgress(progress);
    const sizes = [0, 0, 0];
    const cropX = [0, 0, 0];
    let i;
    if (progress <= 0.00125) {}
    else if (progress <= this.leftEdgeValue) {
      sizes[0] = this.realEdgeWidth * (progress / this.leftEdgeValue);
    } else {
      sizes[0] = this.realEdgeWidth;
      if (progress < this.rightEdgeValue) {
        sizes[1] = tileHeight * (progress - this.leftEdgeValue) / this.middleValue;
      } else {
        sizes[1] = tileHeight;
        if (progress >= 0.99875) {
          sizes[2] = this.realEdgeWidth;
        } else {
          sizes[2] = this.realEdgeWidth * ((progress - this.rightEdgeValue) / this.leftEdgeValue);
        }
      }
    }
    for (i = 0; i < 3; i++) {
      this.filler[i].setCrop(cropX[i], 0, sizes[i], tileHeight);
    }
  }

  setWidth(width: number) {
    this.width = width;
    this.cut = this.width < this.edgeWidth * 2;
    if (this.cut) {
      this.realEdgeWidth = edgeWidth * (this.width / (this.edgeWidth * 2));
      this.middleWidth = 0;
      this.edgeValue = 0.5;
    } else {
      this.realEdgeWidth = edgeWidth;
      this.middleWidth = this.width - this.edgeWidth * 2;
      this.edgeValue = (this.edgeWidth) / this.width;
    }
    this.leftEdgeValue = this.edgeValue
    this.rightEdgeValue = 1 - this.leftEdgeValue;
    this.middleValue = this.rightEdgeValue - this.leftEdgeValue;

    this.left = -Math.ceil(this.width / 2);
    this.right = this.left + this.width;

    this.edges[0].setPosition(this.left, 0);
    this.edges[0].setDisplaySize(this.edgeWidth, this.height);
    this.edges[0].setCrop(0, 0, this.realEdgeWidth, tileHeight);

    this.edges[1].setPosition(this.left + this.edgeWidth, 0);
    this.edges[1].setDisplaySize(this.middleWidth, this.height);
    this.edges[1].setVisible(!this.cut);

    this.edges[2].setPosition(this.right, 0);
    this.edges[2].setDisplaySize(this.edgeWidth, this.height);
    this.edges[2].setCrop(0, 0, this.realEdgeWidth, tileHeight);

    this.filler[0].setPosition(this.left, 0);
    this.filler[0].setDisplaySize(this.edgeWidth, this.height);
    this.filler[0].setCrop(0, 0, this.realEdgeWidth, tileHeight);

    this.filler[1].setPosition(this.left + this.edgeWidth, 0);
    this.filler[1].setDisplaySize(this.middleWidth, this.height);
    this.filler[1].setVisible(!this.cut);

    this.filler[2].setPosition(this.right, 0);
    this.filler[2].setDisplaySize(this.edgeWidth, this.height);
    this.filler[2].setCrop(0, 0, this.realEdgeWidth, tileHeight);

    this.setProgress(null);
  }
}
