import { gameConfig, gameStat, soundConfig } from '../config';
import { soundsController } from './sounds.controller';
import { usageController } from './usage.controller';

class ConfigController {
  resetGameStat() {
    this.setConfig(gameStat, 'started', new Date());
    this.setConfig(gameStat, 'tutorialFinished', false);
    this.setConfig(gameStat, 'score', 0);
    this.setConfig(gameStat, 'clients', 0);
    usageController.log('resetProgress');
  };

  loadConfig() {
    const storage = gameConfig.localStorageName;
    this.setConfig(gameStat, 'id', localStorage[storage + '.id'] || this.uuid());
    this.setConfig(gameStat, 'started', localStorage[storage + '.started'] === undefined ? new Date() : new Date(localStorage[storage + '.started']));
    gameStat.score = parseInt(localStorage[storage + '.score'] || '0');
    gameStat.clients = parseInt(localStorage[storage + '.clients'] || '0');

    gameStat.tutorialFinished = localStorage[storage + '.tutorialFinished'] === 'true';

    soundConfig.musicMuted = localStorage[storage + '.musicMuted'] === 'true';
    soundConfig.soundsMuted = localStorage[storage + '.soundsMuted'] === 'true';
    soundConfig.musicVolume = parseFloat(localStorage[storage + '.musicVolume'] || 1);
    soundConfig.soundsVolume = parseFloat(localStorage[storage + '.soundsVolume'] || 1);
  }

  setConfig(object, field, value) {
    object[field] = value;
    localStorage[gameConfig.localStorageName + '.' + field] = value;
    if (field === 'musicVolume') {
      soundsController.resetMusicVolume();
    }
  }

  uuid(): string {
    // @ts-ignore
    return ([1e7]+-1e3+-4e3+-8e3+-1e11).replace(/[018]/g,c=>(c^crypto.getRandomValues(new Uint8Array(1))[0]&15 >> c/4).toString(16));
  }

}

export const configController = new ConfigController();
