import { loaderConfig, soundConfig } from '../config';
import { configController } from './config.controller';

class SoundsController {
  private soundManager;
  private actionSounds: Record<string, Phaser.Sound.BaseSound>;
  private soundtracks: Record<string, Phaser.Sound.BaseSound>;

  private temporaryMute = false;
  private musicPaused = false;

  constructor() {

  }

  setGame(game) {
    this.soundManager = game.sound;

    this.actionSounds = {};
    this.soundtracks = {};

    for (const sound of loaderConfig.actionSounds) {
      const name = sound.split('.')[0];
      this.actionSounds[name] = this.soundManager.add(name);
    }

    for (const name of loaderConfig.soundtracks) {
      this.soundtracks[name] = this.soundManager.add(name);
    }
  }

  mute(temporary = false) {
    if (temporary && soundConfig.soundsMuted) {
      return;
    }
    this.temporaryMute = temporary;
    if (soundConfig.music && !soundConfig.music.isPaused) {
      soundConfig.music.pause();
    }
    if (!this.temporaryMute) {
      configController.setConfig(soundConfig, 'soundsMuted', true);
    }
  }

  unmute(temporary = false) {
    if (temporary && !this.temporaryMute) {
      return;
    }
    if (soundConfig.music && soundConfig.music.isPaused && !this.musicPaused) {
      soundConfig.music.resume();
    }
    configController.setConfig(soundConfig, 'soundsMuted', false);
    this.temporaryMute = false;
  }

  actionSound(name) {
    if (!soundConfig.soundsMuted && !this.temporaryMute) {
      this.actionSounds[name].play(this.soundParams);
    }
  }

  stopMusic() {
    if (soundConfig.music) {
      soundConfig.music.stop();
      soundConfig.music = null;
    }
  }

  playSoundtrack(name) {
    const exist = this.soundtracks.hasOwnProperty(name);
    if (!exist) {
      this.stopMusic();
    } else if (soundConfig.music !== this.soundtracks[name]) {
      this.stopMusic();
      soundConfig.music = this.soundtracks[name];
      soundConfig.music.play(this.musicParams);
      if (soundConfig.soundsMuted) {
        soundConfig.music.pause();
      }
    }
  }

  pauseMusic(pause) {
    this.musicPaused = pause;
    if (soundConfig.music) {
      if (pause && !soundConfig.music.isPaused) {
        soundConfig.music.pause();
      } else if (!pause && !soundConfig.soundsMuted) {
        soundConfig.music.play();
      }
    }
  }

  resetMusicVolume() {
    if (soundConfig.music) {
      // @ts-ignore
      if (typeof soundConfig.music.setVolume === 'function') {
        // @ts-ignore
        soundConfig.music.setVolume(this.musicParams.volume);
      }
    }
  }

  private get soundParams() {
    const soundParams = Object.assign({}, soundConfig.soundParams);
    if (soundConfig.soundParams.volume !== undefined) {
      soundParams.volume = soundConfig.soundParams.volume * soundConfig.soundsVolume;
    }
    return soundParams;
  }

  private get musicParams() {
    const musicParams = Object.assign({}, soundConfig.musicParams);
    if (soundConfig.musicParams.volume !== undefined) {
      musicParams.volume = soundConfig.musicParams.volume * soundConfig.musicVolume;
    }
    return musicParams;
  }

}


export const soundsController = new SoundsController();
