import EffectSprite from '../sprites/effect.sprite';
import IngredientSprite from '../sprites/ingredient.sprite';
import BaseLevelScene from '../scenes/baseLevel.scene';

class SpritesController {
  effects: EffectSprite[] = [];
  ingredients: IngredientSprite[] = [];

  scene: BaseLevelScene;

  setScene(scene) {
    if (this.scene) {
      for (const effect of this.effects) {
        effect.destroy();
      }
      this.effects = [];
      for (const ingredient of this.ingredients) {
        ingredient.destroy();
      }
      this.ingredients = [];
    }
    this.scene = scene;
  }

  getEffect(type) {
    let effect: EffectSprite;
    if (this.effects.length) {
      effect = this.effects.pop() as EffectSprite;
      effect.setType(type);
    } else {
      effect = new EffectSprite(this.scene, type);
    }
    effect.setAlpha(1);
    return effect;
  }

  releaseEffect(effect: EffectSprite|EffectSprite[]) {
    if (Array.isArray(effect)) {
      for (const item of effect) {
        item.parentContainer?.remove(item);
      }
      this.effects.push(...effect);
    } else {
      effect.parentContainer?.remove(effect);
      this.effects.push(effect);
    }
  }

  getIngredient(type, background = null) {
    let ingredient: IngredientSprite;
    if (this.effects.length) {
      ingredient = this.ingredients.pop() as IngredientSprite;
      ingredient.setType(type).setBackground(background);
    } else {
      ingredient = new IngredientSprite(this.scene, type, background);
    }
    ingredient.setAlpha(1);
    return ingredient;
  }

  releaseIngredient(ingredient: IngredientSprite|IngredientSprite[]) {
    if (Array.isArray(ingredient)) {
      for (const item of ingredient) {
        item.parentContainer?.remove(item);
      }
      this.ingredients.push(...ingredient);
    } else {
      ingredient.parentContainer?.remove(ingredient);
      this.ingredients.push(ingredient);
    }
  }
}

export const spritesController = new SpritesController();
