import { loaderConfig } from '../config';
import BaseScene from './base.scene';
import { soundsController } from '../controllers';
import { game } from '../game';

export default class PreloadScene extends BaseScene {
  percentText;

  constructor () {
    super({ key: 'PreloadScene' });
  }

  preload () {
    this.game.resize();
    setTimeout(() => {
      this.game.resize();
    }, 250);
    this.getWorldView();
    document.getElementById('spinner')?.remove();
    this.percentText = this.make.text({
      x: this.view.centerX,
      y: this.view.centerY,
      text: '0%',
      style: {
        align: 'center',
        fontFamily: '"Architects Daughter"',
        fontSize: '48px',
        color: '#000'
      }
    });
    this.percentText.setOrigin(0.5);
    this.load.on('progress', (value) => {
      this.percentText.setText(parseInt((value * 100).toString()) + '%');
    });

    if (!game) {
      this.percentText.setText('Cannot run game\n\nin Private mode\n\nin Chrome-based browser ;(');
      return;
    }


    this.load.atlas('ui', 'assets/images/uipack_rpg_sheet.png', 'assets/images/uipack_rpg_sheet.json');
    this.load.atlas('clients', 'assets/images/clients.png', 'assets/images/clients.json');
    this.load.atlas('effects', 'assets/images/effects.png', 'assets/images/effects.json');
    this.load.atlas('ingredients', 'assets/images/ingredients.png', 'assets/images/ingredients.json');
    this.load.image('musicOn', 'assets/images/musicOn.png');
    this.load.image('musicOff', 'assets/images/musicOff.png');
    for (let sound of loaderConfig.soundtracks) {
      this.load.audio(sound, 'assets/sounds/' + sound + '.mp3');
    }
    for (let sound of loaderConfig.actionSounds) {
      const name = sound.split('.')[0];
      this.load.audio(name, 'assets/sounds/' + sound);
    }
    //for (let sound of loaderConfig.finishSounds) {
    //  const name = sound.split('.')[0];
    //  this.load.audio(name, 'assets/sounds/' + sound);
    //}
  }

  create () {
    super.create();
    soundsController.setGame(game);

    this.percentText.destroy();
    this.percentText = null;

    this.scene.start('MainMenuScene');
  }

  update () {}

  onResize() {
    super.onResize();
    if (this.percentText) {
      this.percentText.setPosition(this.view.centerX, this.view.centerY);
    }
  }
}
