import Phaser from "phaser";
import BaseLevelScene from '../scenes/baseLevel.scene';
import EffectSprite from './effect.sprite';
import { spritesController } from '../controllers';

const effectMargin = 8;
const effectWidth = 64 + effectMargin;

export default class EffectsContainer extends Phaser.GameObjects.Container {
  scene: BaseLevelScene;
  effects:EffectSprite[] = [];
  effectsMap: Record<string, any> = {};

  constructor(scene, x, y) {
    super(scene, x, y);

    this.scene = scene;
    this.scene.add.existing(this);
  }

  update(time, delta) {
    super.update(time, delta);
  }

  setEffects(effects: string[]) {
    let i, j;
    const removedEffects: EffectSprite[] = [];
    const effectsMap = {};
    for (i = 0; i < effects.length; i++) {
      effectsMap[effects[i]] = 0;
    }
    for (i = 0; i < this.effects.length; i++) {
      if (!effectsMap.hasOwnProperty(this.effects[i].type)) {
        removedEffects.push(this.effects.splice(i, 1)[0] as EffectSprite);
        i--;
      } else {
        effectsMap[this.effects[i].type] = 1;
      }
    }
    this.scene.tweens.add({
      targets: removedEffects,
      alpha: {from: 1, to: 0},
      ease: 'Sine.easeOut',
      duration: 500,
      onComplete: () => {
        spritesController.releaseEffect(removedEffects);
      },
    });
    for (i = 0; i < effects.length; i++) {
      if (!effectsMap[effects[i]]) {
        j = this.effects.length;
        this.effects.push(spritesController.getEffect(effects[i]));
        this.effects[j].setPosition(500, 0).setAlpha(0);
        this.add(this.effects[j]);
      }
    }
    this.effectsMap = effectsMap;
    const width = effectWidth * this.effects.length;
    let x = -Math.ceil((width - effectMargin) / 2);
    for (i = 0; i < this.effects.length; i++) {
      if (this.effects[i].x === 500) {
        this.effects[i].setPosition(x, 0);
        this.scene.tweens.add({
          targets: this.effects[i],
          alpha: 1,
          ease: 'Sine.easeOut',
          duration: 350,
        });

      } else {
        this.scene.tweens.add({
          targets: this.effects[i],
          x,
          ease: 'Sine.easeOut',
          duration: 100,
        });
      }
      x += effectWidth;
    }
  }

  clearEffects() {
    if (!this.effects.length) {
      return;
    }
    spritesController.releaseEffect(this.effects);
    this.effects = [];
    this.effectsMap = {};
  }
}
