import Phaser from 'phaser'
import BaseLevelScene from '../scenes/baseLevel.scene';
import Sprite = Phaser.GameObjects.Sprite;

type Backgroud = 'cornered_light'|'soft_gray'|null;
type Callback = (sprite: IngredientSprite) => void;

export default class IngredientSprite extends Phaser.GameObjects.Container {
  scene: BaseLevelScene;
  type: string;
  background: Backgroud;

  sprite: Sprite;
  backgroundSprite: Sprite;

  callback: Callback;

  constructor(scene, type, background: Backgroud = null, callback: Callback = () => {}) {
    super(scene, 0, 0);

    this.scene  = scene;
    if (type === -1) {
      type = Phaser.Math.Between(0, 7);
    }
    this.type = type;
    this.background = background;
    this.callback = callback;

    this.backgroundSprite = new Phaser.GameObjects.Sprite(scene, 0, 0, 'ingredients',  'background_' + (background || 'cornered_light') + '.png');
    this.backgroundSprite.setDisplaySize(128, 128);
    if (!background) {
      this.backgroundSprite.setAlpha(0);
    }
    this.add(this.backgroundSprite);
    this.sprite = new Phaser.GameObjects.Sprite(scene, 0, 0, 'ingredients', type + '.png');
    this.sprite.setDisplaySize(128, 128);
    this.add(this.sprite);
    this.sprite.setInteractive({useHandCursor: true});
    this.sprite.on('pointerup', () => {this.runCallback();});

    scene.add.existing(this);
  }

  update(time, delta) {
    super.update(time, delta);
    this.sprite.update(time, delta);
  }

  setType(type: string) {
    this.type = type;
    this.sprite.setFrame(type + '.png');
    return this;
  }

  setBackground(background: Backgroud) {
    this.background = background;
    if (background) {
      this.backgroundSprite.setFrame( 'background_' + background + '.png').setAlpha(1);
    } else {
      this.backgroundSprite.setAlpha(0);
    }
    return this;
  }

  runCallback() {
    if (this.background == 'soft_gray') {
      this.setBackground('cornered_light');
      this.callback(this);
    }
  }

}
